package ru.dbotthepony.kstarbound.world.api

import java.io.DataInputStream

sealed class AbstractCell {
	abstract val foreground: AbstractTileState
	abstract val background: AbstractTileState
	abstract val liquid: AbstractLiquidState

	abstract val dungeonId: Int
	abstract val biome: Int
	abstract val envBiome: Int
	abstract val isIndestructible: Boolean

	abstract fun immutable(): ImmutableCell
	abstract fun mutable(): MutableCell

	companion object {
		fun skip(stream: DataInputStream) {
			AbstractTileState.skip(stream)
			AbstractTileState.skip(stream)
			AbstractLiquidState.skip(stream)
			stream.skipNBytes(1 + 2 + 1 + 1 + 1 + 1)
		}

		val EMPTY = ImmutableCell(AbstractTileState.EMPTY, AbstractTileState.EMPTY, AbstractLiquidState.EMPTY, 0, 0, 0, false)
		val NULL = ImmutableCell(AbstractTileState.NULL, AbstractTileState.NULL, AbstractLiquidState.EMPTY, 0, 0, 0, false)
	}
}
